import json
import logging

logger = logging.getLogger('data')


class BaseConsumer(object):
    def __init__(self, conf=None):
        pass

    @classmethod
    def from_config(cls, opt):
        config = opt.get('config')
        return cls(conf=config)

    def stop(self):
        pass


class Printer(BaseConsumer):
    """
    Print the data received on stdout.
    """

    def process(self, msg):
        if isinstance(msg, str):
            logger.debug(msg)
        else:
            logger.debug(json.dumps(msg))


class SessionPrinter(BaseConsumer):
    """
    Print the data received on stdout.
    """

    def __init__(self, conf=None):
        super().__init__(conf)
        # признак что выводим только id изменяемых данных, а не всю запись
        self.only_ids = conf.getboolean('pgdatalogger', 'only_ids')

    def process(self, msg):
        # найдем среди изменений таблицу commit_trigger.trigger_called
        # из нее достанем параметры сессии - они должны быть в каждой транзакции
        session = next((block['record'] for block in msg['changes']
                        if block['table'] == 'trigger_called'), {})

        # уберем начальные подчеркивания, чтобы ELK корректно спарсил данные
        for key in session.keys():
            if key.startswith('_'):
                session[key.replace('_', '', 1)] = session[key]
                del session[key]

        # вытащим данные сессии из строки для парсинга
        if session.get('session'):
            session['session'] = json.loads(session['session'])

        for block in msg['changes']:
            if block['table'] != 'trigger_called':
                if self.only_ids and block['record']:
                    del block['record']

                # создадим итоговую строку как данные сессии + данные блока
                row = session.copy()
                row.update(block)

                if row.get('record'):
                    # для insert подстави id в key, чтобы можно было найти в
                    # истории добавление записи
                    if not row.get('key'):
                        row['key'] = row['record'].get('id')
                    # завернем record в строку, чтобы не наткнуться на
                    # ситуацию, когда поля в разных таблицах имеют одно
                    # наименование, но разные типы
                    row['record'] = json.dumps(row['record'])

                logger.debug(f'{json.dumps(row)}')
