from django.conf import settings
from pgdatalogger.utils import set_session_data


class PGDataLoggerMiddleware:
    """
    Выставление характеристик сессии
    Необходимо подключать после production_request, т.к. берутся теже данные
    """
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        # при подключенном production_request эти параметры будет в request
        request_id = request.production_request_uuid or ''
        # подготовим объект для сохранения данных о сессии
        session_data = {
            'id': request_id,
            'user': request.user.is_authenticated and request.user.pk or None,
            'path': request.path_info.replace(settings.ROOT_URL, ''),
            'is_celery': False,
        }
        set_session_data(session_data)
        return self.get_response(request)
